@@ -33,6 +33,8 @@ class Property(typing.Generic[DeserializedType, SerializedType]):
         self.serializable: bool = serializable
         self.deserializable: bool = deserializable
         self.default: typing.Optional[DeserializedType] = default
+        # Distinguish between no default being provided and the default being None
+        self.optional = False
         self.override = override
 
     @property
@@ -85,11 +87,15 @@ class Property(typing.Generic[DeserializedType, SerializedType]):
         for field in fields:
             next_value = value.get(field)
             if next_value is None:
+                if self.default is None and not self.optional:
+                    msg = "Unable to deserialize {} into {}, missing a required field: {}".format(
+                        data, self.underlying_types, field)
+                    raise RuntimeError(msg)
                 # This occurs if a `field` is unexpectedly not present in the data dictionary
                 # or if its value is null.
                 # Use the default value and stop traversing, even if we have not yet reached
                 # the last field in the serialization path.
-                value = self.default
+                value = self.serialize(self.default)
                 break
             else:
                 value = next_value
@@ -134,6 +140,35 @@ class Property(typing.Generic[DeserializedType, SerializedType]):
         return '<Property {!r}>'.format(self.serialization_path)
 
 
+class PropertyCollection(Property[DeserializedType, SerializedType]):
+
+    def __set__(self, obj, value: typing.Union[SerializedType, DeserializedType]):
+        """
+        Property setter for container property types.
+
+        This setter defers to the subclass to implement the `_set_elements` logic
+        """
+        base_class = _get_base_class(obj, self.serialization_path)
+        if issubclass(type(value), self.underlying_types):
+            value_to_set = self._set_elements(value)
+        else:
+            # if value is not an underlying type, set its deserialized version.
+            value_to_set = self.deserialize(value, base_class=base_class)
+
+        if base_class is not None and self.override:
+            getattr(base_class, self.serialization_path).fset(obj, value_to_set)
+        else:
+            setattr(obj, self._key, value_to_set)
+
+    @abstractmethod
+    def _set_elements(self, value: typing.Union[SerializedType, DeserializedType]):
+        """
+        Perform any needed underlying element specific deserialization.
+
+        Return the appropriate value to set
+        """
+
+
 def _get_base_class(obj: object, key: str) -> type:
     """
     Return the base class that has key as an attribute, if it exists.
@@ -142,7 +177,10 @@ def _get_base_class(obj: object, key: str) -> type:
     with key as an attribute, return None.
     """
     base_classes = obj.__class__.__bases__  # Tuple of all base classes of obj
-    classes_with_key = [base_class for base_class in base_classes if hasattr(base_class, key)]
+    try:
+        classes_with_key = [base_class for base_class in base_classes if hasattr(base_class, key)]
+    except TypeError:
+        return None
     if len(classes_with_key) == 1:
         return classes_with_key[0]
     else:
@@ -304,7 +342,7 @@ class Datetime(Property[datetime, int]):
         return int(arrow.get(value).float_timestamp * 1000)
 
 
-class List(Property[list, list]):
+class List(PropertyCollection[list, list]):
 
     def __init__(self,
                  element_type: typing.Union[Property, typing.Type[Property]],
@@ -322,7 +360,7 @@ class List(Property[list, list]):
 
     @property
     def underlying_types(self):
-        return list
+        return list, set, tuple
 
     @property
     def serialized_types(self):
@@ -340,8 +378,20 @@ class List(Property[list, list]):
             serialized.append(self.element_type.serialize(element))
         return serialized
 
+    def _set_elements(self, value):
+        elems = []
+        for sub_val in value:
+            if isinstance(self.element_type, PropertyCollection):
+                val_to_append = self.element_type._set_elements(sub_val)
+            elif issubclass(type(sub_val), self.element_type.underlying_types):
+                val_to_append = sub_val
+            else:
+                val_to_append = self.element_type.deserialize(sub_val)
+            elems.append(val_to_append)
+        return elems
 
-class Set(Property[set, typing.Iterable]):
+
+class Set(PropertyCollection[set, typing.Iterable]):
 
     def __init__(self,
                  element_type: typing.Union[Property, typing.Type[Property]],
@@ -359,7 +409,7 @@ class Set(Property[set, typing.Iterable]):
 
     @property
     def underlying_types(self):
-        return set
+        return set, list, tuple
 
     @property
     def serialized_types(self):
@@ -380,6 +430,18 @@ class Set(Property[set, typing.Iterable]):
         except TypeError:
             return serialized
 
+    def _set_elements(self, value):
+        elems = set()
+        for sub_val in value:
+            if isinstance(self.element_type, PropertyCollection):
+                val_to_append = self.element_type._set_elements(sub_val)
+            elif issubclass(type(sub_val), self.element_type.underlying_types):
+                val_to_append = sub_val
+            else:
+                val_to_append = self.element_type.deserialize(sub_val)
+            elems.add(val_to_append)
+        return elems
+
 
 class Union(Property[typing.Any, typing.Any]):
     """
@@ -432,11 +494,13 @@ class Union(Property[typing.Any, typing.Any]):
                 return prop.deserialize(value)
             except ValueError:
                 pass
-        raise RuntimeError("An unexpected error occurred while trying to deserialize {} to one "
-                           "of the following types: {}.".format(value, self.underlying_types))
+        # this is a failsafe that shouldn't actually ever get hit, hence no cover
+        msg = "An unexpected error occurred while trying to deserialize {} to one of the " \
+              "following types: {}.".format(value, self.underlying_types)  # pragma: no cover
+        raise ValueError(msg)  # pragma: no cover
 
 
-class SpecifiedMixedList(Property[list, list]):
+class SpecifiedMixedList(PropertyCollection[list, list]):
     """A finite list in which the type of each entry is specified."""
 
     def __init__(self,
@@ -458,7 +522,7 @@ class SpecifiedMixedList(Property[list, list]):
 
     @property
     def underlying_types(self):
-        return list
+        return list, tuple
 
     @property
     def serialized_types(self):
@@ -492,6 +556,26 @@ class SpecifiedMixedList(Property[list, list]):
 
         return serialized
 
+    def _set_elements(self, value):
+        elems = []
+        if len(value) > len(self.element_types):
+            raise ValueError("Cannot serialize value {}, as it has more elements "
+                             "than expected for list {}".format(value, self.element_types))
+        for element, element_type in zip(value, self.element_types):
+            if isinstance(element_type, PropertyCollection):
+                val_to_append = element_type._set_elements(element)
+            elif issubclass(type(element), element_type.underlying_types):
+                val_to_append = element
+            else:
+                val_to_append = element_type.deserialize(element)
+            elems.append(val_to_append)
+
+        # If there are more element types than elements, append serialized default values
+        for element_type in self.element_types[len(value):]:
+            elems.append(element_type.default)
+
+        return elems
+
 
 class Enumeration(Property[BaseEnumeration, str]):
 
@@ -524,7 +608,7 @@ class Enumeration(Property[BaseEnumeration, str]):
         return self.klass.get_value(value)
 
 
-class Object(Property[typing.Any, dict]):
+class Object(PropertyCollection[typing.Any, dict]):
 
     def __init__(self,
                  klass: typing.Type[typing.Any],
@@ -540,7 +624,7 @@ class Object(Property[typing.Any, dict]):
                          override)
         self.klass = klass
         # We need to use __dict__ here because other access methods will invoke __get__
-        # Start with the fields from the parent classes, overwritting with any newer classes
+        # Start with the fields from the parent classes, overwriting with any newer classes
         self.fields = {k: v
                        for x in self.klass.__bases__
                        for k, v in x.__dict__.items() if isinstance(v, Property)}
@@ -601,8 +685,14 @@ class Object(Property[typing.Any, dict]):
     def __str__(self):
         return '<Object[{}] {!r}>'.format(self.klass.__name__, self.serialization_path)
 
+    def _set_elements(self, value):
+        if issubclass(type(value), self.klass):
+            return value
+        else:
+            return self.deserialize(value)
 
-class LinkOrElse(Property[typing.Union[Serializable, LinkByUID], dict]):
+
+class LinkOrElse(PropertyCollection[typing.Union[Serializable, LinkByUID], dict]):
     """
     A property that can either be a serializable object with IDs or a LinkByUID object.
 
@@ -654,8 +744,11 @@ class LinkOrElse(Property[typing.Union[Serializable, LinkByUID], dict]):
         raise Exception("Serializable object that is being pointed to must have a self-contained "
                         "build() method that does not call deserialize().")
 
+    def _set_elements(self, value):
+        return value
 
-class Optional(Property[typing.Optional[typing.Any], typing.Optional[typing.Any]]):
+
+class Optional(PropertyCollection[typing.Optional[typing.Any], typing.Optional[typing.Any]]):
 
     def __init__(self,
                  prop: typing.Union[Property, typing.Type[Property]],
@@ -671,6 +764,7 @@ class Optional(Property[typing.Optional[typing.Any], typing.Optional[typing.Any]
                          default,
                          override)
         self.prop = prop if isinstance(prop, Property) else prop()
+        self.optional = True
 
     @property
     def underlying_types(self):
@@ -697,8 +791,17 @@ class Optional(Property[typing.Optional[typing.Any], typing.Optional[typing.Any]
     def __str__(self):
         return '<Optional[{}] {!r}>'.format(self.prop, self.serialization_path)
 
+    def _set_elements(self, value):
+        elem = None
+        if value is not None:
+            if isinstance(self.prop, PropertyCollection):
+                elem = self.prop._set_elements(value)
+            else:
+                elem = value
+        return elem
 
-class Mapping(Property[dict, dict]):
+
+class Mapping(PropertyCollection[dict, dict]):
     """
     Serialization of a Mapping.
 
@@ -724,6 +827,7 @@ class Mapping(Property[dict, dict]):
                          deserializable,
                          default,
                          override)
+
         self.keys_type = keys_type if isinstance(keys_type, Property) else keys_type()
         self.values_type = values_type if isinstance(values_type, Property) else values_type()
         self.ser_as_list_of_pairs = ser_as_list_of_pairs
@@ -770,3 +874,23 @@ class Mapping(Property[dict, dict]):
             serialized_value = self.values_type.serialize(value)
             serialized[serialized_key] = serialized_value
         return serialized
+
+    def _set_elements(self, value):
+        elems = dict()
+        for key, val in value.items():
+            if isinstance(self.values_type, PropertyCollection):
+                deserialized_value = self.values_type._set_elements(val)
+            elif issubclass(type(val), self.values_type.underlying_types):
+                deserialized_value = val
+            else:
+                deserialized_value = self.values_type.deserialize(val)
+
+            if isinstance(self.keys_type, PropertyCollection):
+                deserialized_key = self.keys_type._set_elements(key)
+            elif issubclass(type(key), self.keys_type.underlying_types):
+                deserialized_key = key
+            else:
+                deserialized_key = self.keys_type.deserialize(key)
+
+            elems[deserialized_key] = deserialized_value
+        return elems
