@@ -1,14 +1,26 @@
 """Resources that represent both individual and collections of datasets."""
 from collections import defaultdict
-from typing import TypeVar, List
+from typing import TypeVar, List, Optional, Union, Tuple, Iterator
 from uuid import UUID
 
+from gemd.entity.base_entity import BaseEntity
+from gemd.entity.link_by_uid import LinkByUID
+from gemd.entity.object import MeasurementSpec, MeasurementRun, MaterialSpec, MaterialRun, \
+    ProcessSpec, ProcessRun, IngredientSpec, IngredientRun
+from gemd.entity.template import PropertyTemplate, MaterialTemplate, MeasurementTemplate, \
+    ParameterTemplate, ProcessTemplate, ConditionTemplate
+from gemd.util import writable_sort_order
+
 from citrine._rest.collection import Collection
-from citrine._rest.resource import Resource
+from citrine._rest.resource import Resource, ResourceTypeEnum
 from citrine._serialization import properties
 from citrine._session import Session
-from citrine._utils.functions import scrub_none
+from citrine._utils.functions import scrub_none, format_escaped_url
+from citrine.exceptions import NotFound
+from citrine.resources.api_error import ApiError
 from citrine.resources.condition_template import ConditionTemplateCollection
+from citrine.resources.data_concepts import _make_link_by_uid
+from citrine.resources.delete import _async_gemd_batch_delete, _poll_for_async_batch_delete_result
 from citrine.resources.file_link import FileCollection
 from citrine.resources.ingredient_run import IngredientRunCollection
 from citrine.resources.ingredient_spec import IngredientSpecCollection
@@ -23,16 +35,6 @@ from citrine.resources.process_run import ProcessRunCollection
 from citrine.resources.process_spec import ProcessSpecCollection
 from citrine.resources.process_template import ProcessTemplateCollection
 from citrine.resources.property_template import PropertyTemplateCollection
-from gemd.entity.object import MeasurementSpec, MeasurementRun, MaterialSpec, MaterialRun, \
-    ProcessSpec, ProcessRun, IngredientSpec, IngredientRun
-from gemd.entity.template.condition_template import ConditionTemplate
-from gemd.entity.template.material_template import MaterialTemplate
-from gemd.entity.template.measurement_template import MeasurementTemplate
-from gemd.entity.template.parameter_template import ParameterTemplate
-from gemd.entity.template.process_template import ProcessTemplate
-from gemd.entity.template.property_template import PropertyTemplate
-from gemd.entity.base_entity import BaseEntity
-from gemd.util import writable_sort_order
 
 ResourceType = TypeVar('ResourceType', bound='DataConcepts')
 
@@ -53,6 +55,8 @@ class Dataset(Resource['Dataset']):
         A summary of this dataset.
     description: str
         Long-form description of the dataset.
+    unique_name: Optional[str]
+        An optional, globally unique name that can be used to retrieve the dataset.
 
     Attributes
     ----------
@@ -78,9 +82,11 @@ class Dataset(Resource['Dataset']):
     """
 
     _response_key = 'dataset'
+    _resource_type = ResourceTypeEnum.DATASET
 
     uid = properties.Optional(properties.UUID(), 'id')
     name = properties.String('name')
+    unique_name = properties.Optional(properties.String(), 'unique_name')
     summary = properties.String('summary')
     description = properties.String('description')
     deleted = properties.Optional(properties.Boolean(), 'deleted')
@@ -92,10 +98,12 @@ class Dataset(Resource['Dataset']):
     delete_time = properties.Optional(properties.Datetime(), 'delete_time')
     public = properties.Optional(properties.Boolean(), 'public')
 
-    def __init__(self, name: str, summary: str, description: str):
+    def __init__(self, name: str, *, summary: str,
+                 description: str, unique_name: Optional[str] = None):
         self.name: str = name
         self.summary: str = summary
         self.description: str = description
+        self.unique_name = unique_name
 
         # The attributes below should not be set by the user. Instead they will be updated as the
         # dataset interacts with the backend data service
@@ -221,12 +229,12 @@ class Dataset(Resource['Dataset']):
         if isinstance(data_concepts_resource, ConditionTemplate):
             return self.condition_templates
 
-    def register(self, data_concepts_resource: ResourceType, dry_run=False) -> ResourceType:
+    def register(self, data_concepts_resource: ResourceType, *, dry_run=False) -> ResourceType:
         """Register a data concepts resource to the appropriate collection."""
         return self._collection_for(data_concepts_resource)\
             .register(data_concepts_resource, dry_run=dry_run)
 
-    def register_all(self, data_concepts_resources: List[ResourceType],
+    def register_all(self, data_concepts_resources: List[ResourceType], *,
                      dry_run=False) -> List[ResourceType]:
         """
         Register multiple data concepts resources to each of their appropriate collections.
@@ -271,13 +279,109 @@ class Dataset(Resource['Dataset']):
                     resources.extend(registered)
         return resources
 
-    def delete(self, data_concepts_resource: ResourceType, dry_run=False) -> ResourceType:
-        """Delete a data concepts resource to the appropriate collection."""
-        uid = next(iter(data_concepts_resource.uids.items()), None)
-        if uid is None:
-            raise ValueError("Only objects that contain identifiers can be deleted.")
+    def update(self, model: ResourceType) -> ResourceType:
+        """Update a data concepts resource using the appropriate collection."""
+        return self._collection_for(model).update(model)
+
+    def delete(self, data_concepts_resource: Union[UUID, str, LinkByUID, ResourceType], *,
+               dry_run=False) -> ResourceType:
+        """
+        Delete a data concepts resource from the appropriate collection.
+
+        Parameters
+        ----------
+        data_concepts_resource: Union[UUID, str, LinkByUID, ResourceType]
+            A representation of the resource to delete (Citrine id, LinkByUID, or the object)
+        dry_run: bool
+            Whether to actually delete the item or run a dry run of the delete operation.
+            Dry run is intended to be used for validation. Default: false
+
+        """
+        link = _make_link_by_uid(data_concepts_resource)
         return self._collection_for(data_concepts_resource) \
-            .delete(uid[1], scope=uid[0], dry_run=dry_run)
+            .delete(link.id, scope=link.scope, dry_run=dry_run)
+
+    def delete_contents(
+        self,
+        *,
+        timeout: float = 2 * 60,
+        polling_delay: float = 1.0
+    ):
+        """
+        Delete all the GEMD objects from within a single Dataset.
+
+        Parameters
+        ----------
+        timeout: float
+            Amount of time to wait on the job (in seconds) before giving up.
+            Note that this number has no effect on the underlying job itself,
+            which can also time out server-side.
+
+        polling_delay: float
+            How long to delay between each polling retry attempt.
+
+        Returns
+        -------
+        List[Tuple[LinkByUID, ApiError]]
+            A list of (LinkByUID, api_error) for each failure to delete an object.
+            Note that this method doesn't raise an exception if an object fails to be
+            deleted.
+
+        """
+        path = format_escaped_url('projects/{project_id}/datasets/{dataset_uid}/contents',
+                                  dataset_uid=self.uid,
+                                  project_id=self.project_id
+                                  )
+
+        response = self.session.delete_resource(path)
+        job_id = response["job_id"]
+
+        return _poll_for_async_batch_delete_result(self.project_id, self.session, job_id, timeout,
+                                                   polling_delay)
+
+    def gemd_batch_delete(
+            self,
+            id_list: List[Union[LinkByUID, UUID, str, BaseEntity]],
+            *,
+            timeout: float = 2 * 60,
+            polling_delay: float = 1.0
+    ) -> List[Tuple[LinkByUID, ApiError]]:
+        """
+        Remove a set of GEMD objects.
+
+        You may provide GEMD objects that reference each other, and the objects
+        will be removed in the appropriate order.
+
+        A failure will be returned if the object cannot be deleted due to an external
+        reference.
+
+        All data objects must be associated with this dataset resource. You must also
+        have write access on this dataset.
+
+        If you wish to delete more than 50 objects, queuing of deletes requires that
+        the types of objects be known, and thus you _must_ provide ids in the form
+        of BaseEntities.
+
+        Also note that Attribute Templates cannot be deleted at present.
+
+        Parameters
+        ----------
+        id_list: List[Union[LinkByUID, UUID, str, BaseEntity]]
+            A list of the IDs of data objects to be removed. They can be passed
+            as a LinkByUID tuple, a UUID, a string, or the object itself. A UUID
+            or string is assumed to be a Citrine ID, whereas a LinkByUID or
+            BaseEntity can also be used to provide an external ID.
+
+        Returns
+        -------
+        List[Tuple[LinkByUID, ApiError]]
+            A list of (LinkByUID, api_error) for each failure to delete an object.
+            Note that this method doesn't raise an exception if an object fails to be
+            deleted.
+
+        """
+        return _async_gemd_batch_delete(id_list, self.project_id, self.session,
+                                        self.uid, timeout=timeout, polling_delay=polling_delay)
 
 
 class DatasetCollection(Collection[Dataset]):
@@ -324,7 +428,10 @@ class DatasetCollection(Collection[Dataset]):
 
     def register(self, model: Dataset) -> Dataset:
         """
-        Create a new element of the collection by registering an existing resource.
+        Create a new dataset in the collection, or update an existing one.
+
+        If the Dataset has an ID present, then we update the existing resource,
+        else we create a new one.
 
         This differs from super().register() in that None fields are scrubbed, and the json
         response is not assumed to come in a dictionary with a single entry 'dataset'.
@@ -345,7 +452,64 @@ class DatasetCollection(Collection[Dataset]):
         path = self._get_path()
         dumped_dataset = model.dump()
         dumped_dataset["deleted"] = None
-        data = self.session.post_resource(path, scrub_none(dumped_dataset))
+
+        # Only use the idempotent put approach if a) a unique name is provided, and b)
+        # the session is configured to use it (default to False for backwards compatibility).
+        if model.unique_name is not None and self.session.use_idempotent_dataset_put:
+            # Leverage the create-or-update endpoint if we've got a unique name
+            data = self.session.put_resource(path, scrub_none(dumped_dataset))
+        else:
+
+            if model.uid is None:
+                # POST to create a new one if a UID is not assigned
+                data = self.session.post_resource(path, scrub_none(dumped_dataset))
+
+            else:
+                # Otherwise PUT to update it
+                data = self.session.put_resource(
+                    self._get_path(model.uid), scrub_none(dumped_dataset))
+
         full_model = self.build(data)
         full_model.project_id = self.project_id
         return full_model
+
+    def list(self, *,
+             page: Optional[int] = None,
+             per_page: int = 1000) -> Iterator[Dataset]:
+        """
+        List datasets using pagination.
+
+        Leaving page and per_page as default values will yield all elements in the
+        collection, paginating over all available pages.
+
+        Parameters
+        ---------
+        page: int, optional
+            The "page" of results to list. Default is to read all pages and yield
+            all results.  This option is deprecated.
+        per_page: int, optional
+            Max number of results to return per page. Default is 1000.  This parameter
+            is used when making requests to the backend service.  If the page parameter
+            is specified it limits the maximum number of elements in the response.
+
+        Returns
+        -------
+        Iterator[Dataset]
+            Datasets in this collection.
+
+        """
+        return super().list(page=page, per_page=per_page)
+
+    def get_by_unique_name(self, unique_name: str) -> ResourceType:
+        """Get a Dataset with the given unique name."""
+        if unique_name is None:
+            raise ValueError("You must supply a unique_name")
+        path = self._get_path() + "?unique_name=" + unique_name
+        data = self.session.get_resource(path)
+
+        if len(data) == 1:
+            return self.build(data[0])
+        elif len(data) > 1:
+            raise RuntimeError("Received multiple results when requesting a unique dataset")
+        else:
+            raise NotFound(path)
