@@ -6,9 +6,10 @@ from citrine._rest.resource import Resource
 from citrine._serialization.properties import List as PropertyList
 from citrine._serialization.properties import Mapping, String, LinkOrElse, Object
 from citrine._serialization.properties import Optional as PropertyOptional
-from citrine.resources.data_concepts import DataConcepts
+from citrine.resources.data_concepts import DataConcepts, _make_link_by_uid
 from citrine.resources.object_runs import ObjectRun, ObjectRunCollection
 from gemd.entity.file_link import FileLink
+from gemd.entity.link_by_uid import LinkByUID
 from gemd.entity.object.ingredient_run import IngredientRun as GEMDIngredientRun
 from gemd.entity.object.ingredient_spec import IngredientSpec as GEMDIngredientSpec
 from gemd.entity.object.material_run import MaterialRun as GEMDMaterialRun
@@ -50,12 +51,6 @@ class IngredientRun(ObjectRun, Resource['IngredientRun'], GEMDIngredientRun):
     absolute_quantity: :py:class:`ContinuousValue \
     <gemd.entity.value.continuous_value.ContinuousValue>`, optional
         The absolute quantity of the ingredient in the process.
-    name: str, optional
-        Label on the ingredient that is unique within the process that contains it. This property
-        will be overwritten by its value in `spec` if it is present.
-    labels: List[str], optional
-        Additional labels on the ingredient. This property will be overwritten by its value in
-        `spec` if it is present.
     spec: IngredientSpec
         The specification of which this ingredient is a realization.
     file_links: List[FileLink], optional
@@ -65,24 +60,24 @@ class IngredientRun(ObjectRun, Resource['IngredientRun'], GEMDIngredientRun):
 
     _response_key = GEMDIngredientRun.typ  # 'ingredient_run'
 
-    uids = Mapping(String('scope'), String('id'), 'uids')
-    tags = PropertyOptional(PropertyList(String()), 'tags')
-    notes = PropertyOptional(String(), 'notes')
-    material = PropertyOptional(LinkOrElse(), 'material')
-    process = PropertyOptional(LinkOrElse(), 'process')
-    mass_fraction = PropertyOptional(Object(ContinuousValue), 'mass_fraction')
-    volume_fraction = PropertyOptional(Object(ContinuousValue), 'volume_fraction')
-    number_fraction = PropertyOptional(Object(ContinuousValue), 'number_fraction')
+    uids = Mapping(String('scope'), String('id'), 'uids', override=True)
+    tags = PropertyOptional(PropertyList(String()), 'tags', override=True)
+    notes = PropertyOptional(String(), 'notes', override=True)
+    material = PropertyOptional(LinkOrElse(), 'material', override=True)
+    process = PropertyOptional(LinkOrElse(), 'process', override=True)
+    mass_fraction = PropertyOptional(Object(ContinuousValue), 'mass_fraction', override=True)
+    volume_fraction = PropertyOptional(Object(ContinuousValue), 'volume_fraction', override=True)
+    number_fraction = PropertyOptional(Object(ContinuousValue), 'number_fraction', override=True)
     absolute_quantity = PropertyOptional(
-        Object(ContinuousValue), 'absolute_quantity')
-    name = PropertyOptional(String(), 'name')
-    labels = PropertyOptional(PropertyList(String()), 'labels')
-    spec = PropertyOptional(LinkOrElse(), 'spec')
-    file_links = PropertyOptional(PropertyList(Object(FileLink)), 'file_links')
+        Object(ContinuousValue), 'absolute_quantity', override=True)
+    name = PropertyOptional(String(), 'name', override=True)
+    labels = PropertyOptional(PropertyList(String()), 'labels', override=True)
+    spec = PropertyOptional(LinkOrElse(), 'spec', override=True)
+    file_links = PropertyOptional(PropertyList(Object(FileLink)), 'file_links', override=True)
     typ = String('type')
 
     def __init__(self,
-                 name: Optional[str] = None,
+                 *,
                  uids: Optional[Dict[str, str]] = None,
                  tags: Optional[List[str]] = None,
                  notes: Optional[str] = None,
@@ -92,7 +87,6 @@ class IngredientRun(ObjectRun, Resource['IngredientRun'], GEMDIngredientRun):
                  volume_fraction: Optional[ContinuousValue] = None,
                  number_fraction: Optional[ContinuousValue] = None,
                  absolute_quantity: Optional[ContinuousValue] = None,
-                 labels: Optional[List[str]] = None,
                  spec: Optional[GEMDIngredientSpec] = None,
                  file_links: Optional[List[FileLink]] = None):
         if uids is None:
@@ -102,8 +96,8 @@ class IngredientRun(ObjectRun, Resource['IngredientRun'], GEMDIngredientRun):
                                    material=material, process=process,
                                    mass_fraction=mass_fraction, volume_fraction=volume_fraction,
                                    number_fraction=number_fraction,
-                                   absolute_quantity=absolute_quantity, labels=labels,
-                                   name=name, spec=spec, file_links=file_links)
+                                   absolute_quantity=absolute_quantity,
+                                   spec=spec, file_links=file_links)
 
     def __str__(self):
         return '<Ingredient run {!r}>'.format(self.name)
@@ -123,57 +117,71 @@ class IngredientRunCollection(ObjectRunCollection[IngredientRun]):
         """Return the resource type in the collection."""
         return IngredientRun
 
-    def list_by_spec(self, uid: Union[UUID, str], scope: str = 'id') -> Iterator[IngredientRun]:
+    def list_by_spec(self,
+                     uid: Union[UUID, str, LinkByUID, GEMDIngredientSpec], *,
+                     scope: Optional[str] = None) -> Iterator[IngredientRun]:
         """
         [ALPHA] Get the ingredient runs using the specified ingredient spec.
 
         Parameters
         ----------
-        uid
-            The unique ID of the ingredient spec whose ingredient run usages are to be located.
-        scope
-            The scope of `uid`.
+        uid: Union[UUID, str, LinkByUID, GEMDIngredientSpec]
+            A representation of the ingredient spec whose ingredient run usages are to be located.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid, defaults to Citrine scope ("id")
+
         Returns
         -------
         Iterator[IngredientRun]
             The ingredient runs using the specified ingredient spec.
 
         """
-        return self._get_relation('ingredient-specs', uid=uid, scope=scope)
+        link = _make_link_by_uid(uid, scope)
+        return self._get_relation('ingredient-specs', uid=link.id, scope=link.scope)
 
-    def list_by_process(self, uid: Union[UUID, str], scope: str = 'id') -> Iterator[IngredientRun]:
+    def list_by_process(self,
+                        uid: Union[UUID, str, LinkByUID, GEMDProcessRun], *,
+                        scope: Optional[str] = None) -> Iterator[IngredientRun]:
         """
         [ALPHA] Get ingredients to a process.
 
         Parameters
         ----------
-        uid
-            The unique ID of the process whose ingredients are to be located.
-        scope
-            The scope of `uid`.
+        uid: Union[UUID, str, LinkByUID, GEMDProcessRun]
+            A representation of the process whose ingredients are to be located.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid, defaults to Citrine scope ("id")
+
         Returns
         -------
         Iterator[IngredientRun]
             The ingredients to the specified process.
 
         """
-        return self._get_relation(relation='process-runs', uid=uid, scope=scope)
+        link = _make_link_by_uid(uid, scope)
+        return self._get_relation(relation='process-runs', uid=link.id, scope=link.scope)
 
-    def list_by_material(self, uid: Union[UUID, str],
-                         scope: str = 'id') -> Iterator[IngredientRun]:
+    def list_by_material(self,
+                         uid: Union[UUID, str, LinkByUID, GEMDMaterialRun], *,
+                         scope: Optional[str] = None) -> Iterator[IngredientRun]:
         """
         [ALPHA] Get ingredients using the specified material.
 
         Parameters
         ----------
-        uid
-            The unique ID of the material whose ingredient usages are to be located.
-        scope
-            The scope of `uid`.
+        uid: Union[UUID, str, LinkByUID, GEMDMaterialRun]
+            A representation of the material whose ingredient run usages are to be located.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid, defaults to Citrine scope ("id")
+
         Returns
         -------
         Iterator[IngredientRun]
             The ingredients using the specified material
 
         """
-        return self._get_relation(relation='material-runs', uid=uid, scope=scope)
+        link = _make_link_by_uid(uid, scope)
+        return self._get_relation(relation='material-runs', uid=link.id, scope=link.scope)
