@@ -6,9 +6,10 @@ from citrine._rest.resource import Resource
 from citrine._serialization.properties import List as PropertyList
 from citrine._serialization.properties import Mapping, String, LinkOrElse, Object
 from citrine._serialization.properties import Optional as PropertyOptional
-from citrine.resources.data_concepts import DataConcepts, CITRINE_SCOPE
+from citrine.resources.data_concepts import DataConcepts, _make_link_by_uid
 from citrine.resources.object_specs import ObjectSpec, ObjectSpecCollection
 from gemd.entity.file_link import FileLink
+from gemd.entity.link_by_uid import LinkByUID
 from gemd.entity.object.ingredient_spec import IngredientSpec as GEMDIngredientSpec
 from gemd.entity.object.material_spec import MaterialSpec as GEMDMaterialSpec
 from gemd.entity.object.process_spec import ProcessSpec as GEMDProcessSpec
@@ -119,41 +120,47 @@ class IngredientSpecCollection(ObjectSpecCollection[IngredientSpec]):
         return IngredientSpec
 
     def list_by_process(self,
-                        uid: Union[UUID, str],
-                        scope: str = CITRINE_SCOPE) -> Iterator[IngredientSpec]:
+                        uid: Union[UUID, str, LinkByUID, GEMDProcessSpec], *,
+                        scope: Optional[str] = None) -> Iterator[IngredientSpec]:
         """
         [ALPHA] Get ingredients to a process.
 
         Parameters
         ----------
-        uid
-            The unique ID of the process whose ingredients are to be located.
-        scope
-            The scope of `uid`.
+        uid: Union[UUID, str, LinkByUID, GEMDProcessSpec]
+            A representation of the process spec whose ingredients are to be located.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid, defaults to Citrine scope ("id")
+
         Returns
         -------
         Iterator[IngredientSpec]
             The ingredients to the specified process.
 
         """
-        return self._get_relation(relation='process-specs', uid=uid, scope=scope)
+        link = _make_link_by_uid(uid, scope)
+        return self._get_relation(relation='process-specs', uid=link.id, scope=link.scope)
 
     def list_by_material(self,
-                         uid: Union[UUID, str],
-                         scope: str = CITRINE_SCOPE) -> Iterator[IngredientSpec]:
+                         uid: Union[UUID, str, LinkByUID, GEMDMaterialSpec], *,
+                         scope: Optional[str] = None) -> Iterator[IngredientSpec]:
         """
         [ALPHA] Get ingredients using the specified material.
 
         Parameters
         ----------
-        uid
-            The unique ID of the material whose ingredient usages are to be located.
-        scope
-            The scope of `uid`.
+        uid: Union[UUID, str, LinkByUID, GEMDMaterialSpec]
+            A representation of the material spec whose ingredient usages are to be located.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid, defaults to Citrine scope ("id")
+
         Returns
         -------
         Iterator[IngredientSpec]
             The ingredients using the specified material
 
         """
-        return self._get_relation(relation='material-specs', uid=uid, scope=scope)
+        link = _make_link_by_uid(uid, scope)
+        return self._get_relation(relation='material-specs', uid=link.id, scope=link.scope)
