@@ -5,24 +5,23 @@ from logging import getLogger
 from typing import List, Dict, Optional, Type, Iterator, Union
 from uuid import UUID
 
-import deprecation
-
 from citrine._rest.resource import Resource
 from citrine._serialization.properties import List as PropertyList
 from citrine._serialization.properties import Optional as PropertyOptional
 from citrine._serialization.properties import String, LinkOrElse, Mapping, Object
-from citrine.resources.data_concepts import DataConcepts
+from citrine._utils.functions import format_escaped_url
+from citrine.resources.data_concepts import DataConcepts, _make_link_by_uid
 from citrine.resources.material_spec import MaterialSpecCollection
 from citrine.resources.object_runs import ObjectRun, ObjectRunCollection
 from gemd.entity.file_link import FileLink
 from gemd.entity.link_by_uid import LinkByUID
 from gemd.entity.object.material_run import MaterialRun as GEMDMaterialRun
 from gemd.entity.object.material_spec import MaterialSpec as GEMDMaterialSpec
+from gemd.entity.template.material_template import MaterialTemplate as GEMDMaterialTemplate
 from gemd.entity.object.process_run import ProcessRun as GEMDProcessRun
 from gemd.json import GEMDEncoder
 from gemd.util import writable_sort_order
 
-
 logger = getLogger(__name__)
 
 
@@ -64,18 +63,19 @@ class MaterialRun(ObjectRun, Resource['MaterialRun'], GEMDMaterialRun):
 
     _response_key = GEMDMaterialRun.typ  # 'material_run'
 
-    name = String('name')
-    uids = Mapping(String('scope'), String('id'), 'uids')
-    tags = PropertyOptional(PropertyList(String()), 'tags')
-    notes = PropertyOptional(String(), 'notes')
-    process = PropertyOptional(LinkOrElse(), 'process')
-    sample_type = String('sample_type')
-    spec = PropertyOptional(LinkOrElse(), 'spec')
-    file_links = PropertyOptional(PropertyList(Object(FileLink)), 'file_links')
+    name = String('name', override=True)
+    uids = Mapping(String('scope'), String('id'), 'uids', override=True)
+    tags = PropertyOptional(PropertyList(String()), 'tags', override=True)
+    notes = PropertyOptional(String(), 'notes', override=True)
+    process = PropertyOptional(LinkOrElse(), 'process', override=True)
+    sample_type = String('sample_type', override=True)
+    spec = PropertyOptional(LinkOrElse(), 'spec', override=True)
+    file_links = PropertyOptional(PropertyList(Object(FileLink)), 'file_links', override=True)
     typ = String('type')
 
     def __init__(self,
                  name: str,
+                 *,
                  uids: Optional[Dict[str, str]] = None,
                  tags: Optional[List[str]] = None,
                  notes: Optional[str] = None,
@@ -109,21 +109,23 @@ class MaterialRunCollection(ObjectRunCollection[MaterialRun]):
         """Return the resource type in the collection."""
         return MaterialRun
 
-    def get_history(self, scope, id) -> Type[MaterialRun]:
+    def get_history(self, *, scope: Optional[str],
+                    id: Union[str, UUID, LinkByUID, MaterialRun]) -> Type[MaterialRun]:
         """
-        Get the history associated with a material.
+        Get the history associated with a terminal material.
 
         The history contains every single every process, ingredient and material that went into
-        the root material as well as the measurements that were performed on all of those
+        the terminal material as well as the measurements that were performed on all of those
         materials. The returned object is a material run with all of its fields fully populated.
 
         Parameters
         ----------
-        scope: str
-            The scope used to locate the material.
-        id: str
-            The unique id corresponding to `scope`. The lookup will be most efficient if you use
-            the Citrine ID (scope='id') of the material.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid. The lookup will be most efficient if you use the Citrine ID
+            of the material, which is the default if scope=None.
+        id: Union[UUID, str, LinkByUID, MaterialRun]
+            A representation of the material whose history is to be retrieved
 
         Returns
         -------
@@ -133,8 +135,9 @@ class MaterialRunCollection(ObjectRunCollection[MaterialRun]):
             of the object.
 
         """
+        link = _make_link_by_uid(id, scope)
         base_path = os.path.dirname(self._get_path(ignore_dataset=True))
-        path = base_path + "/material-history/{}/{}".format(scope, id)
+        path = base_path + format_escaped_url("/material-history/{}/{}", link.scope, link.id)
         data = self.session.get_resource(path)
 
         # Add the root to the context and sort by writable order
@@ -143,106 +146,77 @@ class MaterialRunCollection(ObjectRunCollection[MaterialRun]):
             data['context'] + [data['root']],
             key=lambda x: writable_sort_order(x["type"])
         )
-        root_uid_scope, root_uid_id = next(iter(data['root']['uids'].items()))
+        terminal_scope, terminal_id = next(iter(data['root']['uids'].items()))
         # Add a link to the root as the "object"
-        blob["object"] = LinkByUID(scope=root_uid_scope, id=root_uid_id)
+        blob["object"] = LinkByUID(scope=terminal_scope, id=terminal_id)
 
         # Serialize using normal json (with the GEMDEncoder) and then deserialize with the
         # GEMDEncoder encoder in order to rebuild the material history
         return MaterialRun.get_json_support().loads(
             json.dumps(blob, cls=GEMDEncoder, sort_keys=True))
 
-    def get_by_process(self, uid: Union[UUID, str], scope: str = 'id') -> Optional[MaterialRun]:
+    def get_by_process(self,
+                       uid: Union[UUID, str, LinkByUID, GEMDProcessRun], *,
+                       scope: Optional[str] = None) -> Optional[MaterialRun]:
         """
         [ALPHA] Get output material of a process.
 
         Parameters
         ----------
-        uid
-            The unique ID of the process whose output is to be located.
-        scope
-            The scope of `uid`.
+        uid: Union[UUID, str, LinkByUID, GEMDProcessRun]
+            A representation of the process whose output is to be located.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid, defaults to Citrine scope ("id")
+
         Returns
         -------
         MaterialRun
             The output material of the specified process, or None if no such material exists.
 
         """
+        link = _make_link_by_uid(uid, scope)
         return next(
-            self._get_relation(relation='process-runs', uid=uid, scope=scope, per_page=1), None)
+            self._get_relation(relation='process-runs', uid=link.id, scope=link.scope, per_page=1),
+            None
+        )
 
-    @deprecation.deprecated(details='Use list_by_spec instead.')
-    def filter_by_spec(self,
-                       spec_id: str,
-                       spec_scope: str = 'id',
-                       per_page: int = None) -> Iterator[MaterialRun]:
-        """
-        [ALPHA] Get all material runs associated with a material spec.
-
-        The material spec is specified by its scope and id.
-
-        :param spec_id: The unique id corresponding to `scope`.
-            The lookup will be most efficient if you use the Citrine ID (scope='id')
-            of the material spec.
-        :param spec_scope: The scope used to locate the material spec.
-        :param per_page: The number of results to return per page.
-        :return: A search result of material runs
-        """
-        if per_page is not None:
-            logger.warning('The per_page parameter will be ignored. Please remove it.')
-        return self.list_by_spec(uid=spec_id, scope=spec_scope)
-
-    @deprecation.deprecated(details='Use list_by_template instead.')
-    def filter_by_template(self,
-                           template_id: str,
-                           template_scope: str = 'id',
-                           per_page: int = None) -> Iterator[MaterialRun]:
-        """
-        [ALPHA] Get all material runs associated with a material template.
-
-        The material template is specified by its scope and id.
-
-        :param template_id: The unique id corresponding to `scope`.
-            The lookup will be most efficient if you use the Citrine ID (scope='id')
-            of the material template.
-        :param template_scope: The scope used to locate the material template.
-        :param per_page: The number of results to return per page.
-            Also used for intermediate queries.
-        :return: A search result of material runs
-        """
-        if per_page is not None:
-            logger.warning('The per_page parameter will be ignored. Please remove it.')
-        return self.list_by_template(uid=template_id, scope=template_scope)
-
-    def list_by_spec(self, uid: Union[UUID, str], scope: str = 'id') -> Iterator[MaterialRun]:
+    def list_by_spec(self,
+                     uid: Union[UUID, str, LinkByUID, GEMDMaterialSpec], *,
+                     scope: Optional[str] = None) -> Iterator[MaterialRun]:
         """
         [ALPHA] Get the material runs using the specified material spec.
 
         Parameters
         ----------
-        uid
-            The unique ID of the material spec whose material run usages are to be located.
-        scope
-            The scope of `uid`.
+        uid: Union[UUID, str, LinkByUID, GEMDMaterialSpec]
+            A representation of the material spec whose material run usages are to be located.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid, defaults to Citrine scope ("id")
+
         Returns
         -------
         Iterator[MaterialRun]
             The material runs using the specified material spec.
 
         """
-        return self._get_relation('material-specs', uid=uid, scope=scope)
+        link = _make_link_by_uid(uid, scope)
+        return self._get_relation('material-specs', uid=link.id, scope=link.scope)
 
-    def list_by_template(self, uid: Union[UUID, str],
-                         scope: str = 'id') -> Iterator[MaterialRun]:
+    def list_by_template(self, uid: Union[UUID, str, LinkByUID, GEMDMaterialTemplate], *,
+                         scope: Optional[str] = None) -> Iterator[MaterialRun]:
         """
         [ALPHA] Get the material runs using the specified material template.
 
         Parameters
         ----------
-        uid
-            The unique ID of the material template whose material run usages are to be located.
-        scope
-            The scope of `uid`.
+        uid: Union[UUID, str, LinkByUID, GEMDMaterialTemplate]
+            A representation of the material template whose material run usages are to be located.
+        scope: Optional[str]
+            [DEPRECATED] use a LinkByUID to specify a custom scope
+            The scope of the uid, defaults to Citrine scope ("id")
+
         Returns
         -------
         Iterator[MaterialRun]
@@ -250,6 +224,6 @@ class MaterialRunCollection(ObjectRunCollection[MaterialRun]):
 
         """
         spec_collection = MaterialSpecCollection(self.project_id, self.dataset_id, self.session)
-        specs = spec_collection.list_by_template(uid=uid, scope=scope)
-        return (run for runs in (self.list_by_spec(spec.uids['id']) for spec in specs)
+        specs = spec_collection.list_by_template(uid=_make_link_by_uid(uid, scope))
+        return (run for runs in (self.list_by_spec(spec) for spec in specs)
                 for run in runs)
